from collections.abc import Sequence
from typing import Annotated, Literal

import pydantic

from geodantic.base import _GeoJSONObject
from geodantic.types import (
    GeoJSONObjectType,
    LineStringCoordinates,
    PolygonCoordinates,
    Position,
)


class Point(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.POINT]
    coordinates: Position


class MultiPoint(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.MULTI_POINT]
    coordinates: Sequence[Position]


class LineString(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.LINE_STRING]
    coordinates: LineStringCoordinates


class MultiLineString(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.MULTI_LINE_STRING]
    coordinates: Sequence[LineStringCoordinates]


class Polygon(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.POLYGON]
    coordinates: PolygonCoordinates


class MultiPolygon(_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.MULTI_POLYGON]
    coordinates: Sequence[PolygonCoordinates]


class GeometryCollection[GeometryT: "Geometry"](_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.GEOMETRY_COLLECTION]
    geometries: Sequence[
        Annotated[
            GeometryT,
            pydantic.Field(discriminator="type"),
        ]
    ]


type Geometry = (
    Point
    | MultiPoint
    | LineString
    | MultiLineString
    | Polygon
    | MultiPolygon
    | GeometryCollection
)



from collections.abc import Mapping, Sequence
from typing import Annotated, Any, Literal

import pydantic

from geodantic.base import _GeoJSONObject
from geodantic.geometries import Geometry
from geodantic.types import GeoJSONObjectType


class Feature[
    GeometryT: Geometry | None,
    PropertiesT: Mapping[str, Any] | pydantic.BaseModel | None,
](_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.FEATURE]
    geometry: Annotated[GeometryT, pydantic.Field(discriminator="type")]
    properties: PropertiesT
    id: str | int | None = None

    @pydantic.field_validator("id")
    @classmethod
    def _id_is_not_none(cls, value: Any) -> Any:
        # This validator will only run if the id was provided
        if value is None:
            raise ValueError("id cannot be None if present")
        return value


class FeatureCollection[
    GeometryT: Geometry | None,
    PropertiesT: Mapping[str, Any] | pydantic.BaseModel | None,
](_GeoJSONObject, frozen=True):
    type: Literal[GeoJSONObjectType.FEATURE_COLLECTION]
    features: Sequence[Feature[GeometryT, PropertiesT]]



from .features import Feature, FeatureCollection
from .geometries import (
    Geometry,
    GeometryCollection,
    LineString,
    MultiLineString,
    MultiPoint,
    MultiPolygon,
    Point,
    Polygon,
)
from .types import (
    BoundingBox,
    BoundingBox2D,
    BoundingBox3D,
    GeoJSONObjectType,
    Latitude,
    LinearRing,
    LineStringCoordinates,
    Longitude,
    PolygonCoordinates,
    Position,
    Position2D,
    Position3D,
)

__all__ = [
    "BoundingBox",
    "BoundingBox2D",
    "BoundingBox3D",
    "Feature",
    "FeatureCollection",
    "GeoJSONObjectType",
    "Geometry",
    "GeometryCollection",
    "Latitude",
    "LinearRing",
    "LineString",
    "LineStringCoordinates",
    "Longitude",
    "MultiLineString",
    "MultiPoint",
    "MultiPolygon",
    "Point",
    "Polygon",
    "PolygonCoordinates",
    "Position",
    "Position2D",
    "Position3D",
]



from abc import ABC
from typing import Any

import pydantic

from geodantic.types import BoundingBox, GeoJSONObjectType


class _GeoJSONObject(pydantic.BaseModel, ABC, frozen=True):
    type: GeoJSONObjectType
    bbox: BoundingBox | None = None

    @pydantic.field_validator("bbox")
    @classmethod
    def _bbox_is_not_none(cls, bbox: Any) -> Any:
        # This validator will only run if the bbox was provided
        if bbox is None:
            raise ValueError("bbox cannot be None if present")
        return bbox



from collections.abc import Sequence
from enum import StrEnum
from typing import Annotated

import annotated_types as at

type Longitude = Annotated[float, at.Ge(-180), at.Le(180)]
type Latitude = Annotated[float, at.Ge(-90), at.Le(90)]

type Position2D = tuple[Longitude, Latitude]
type Position3D = tuple[Longitude, Latitude, float]
type Position = Position2D | Position3D


def _validate_bbox(bbox: tuple[float, ...]) -> bool:
    middle = len(bbox) // 2
    return bbox[0] <= bbox[middle] and bbox[1] <= bbox[middle + 1]


type BoundingBox2D = Annotated[
    tuple[Longitude, Latitude, Longitude, Latitude],
    at.Predicate(_validate_bbox),
]

type BoundingBox3D = Annotated[
    tuple[Longitude, Latitude, float, Longitude, Latitude, float],
    at.Predicate(_validate_bbox),
]

type BoundingBox = BoundingBox2D | BoundingBox3D


def _validate_linear_ring(ring: Sequence[Position]) -> bool:
    return ring[0] == ring[-1]


type LinearRing = Annotated[
    Sequence[Position],
    at.MinLen(4),
    at.Predicate(_validate_linear_ring),
]

type LineStringCoordinates = Annotated[Sequence[Position], at.MinLen(2)]
type PolygonCoordinates = Sequence[LinearRing]


class GeoJSONObjectType(StrEnum):
    POINT = "Point"
    MULTI_POINT = "MultiPoint"
    LINE_STRING = "LineString"
    MULTI_LINE_STRING = "MultiLineString"
    POLYGON = "Polygon"
    MULTI_POLYGON = "MultiPolygon"
    GEOMETRY_COLLECTION = "GeometryCollection"
    FEATURE = "Feature"
    FEATURE_COLLECTION = "FeatureCollection"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['geodantic']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=2.0.3,<3.0.0']

setup_kwargs = {
    'name': 'geodantic',
    'version': '0.3.0',
    'description': 'GeoJSON parsing and validation using Pydantic',
    'long_description': '# geodantic\n\n[![pypi](https://img.shields.io/pypi/v/geodantic.svg)](https://pypi.python.org/pypi/geodantic)\n[![versions](https://img.shields.io/pypi/pyversions/geodantic.svg)](https://github.com/alexandermalyga/geodantic)\n\nLightweight, type-safe and spec-conforming [GeoJSON](https://datatracker.ietf.org/doc/html/rfc7946) parsing and validation using Pydantic.\n\n## Installation\n\n```\npip install geodantic\n```\n\n## Examples\n\nParse and validate GeoJSON features with custom properties:\n\n```python\nimport pydantic\nfrom geodantic import Feature, Point\n\n# Use your own properties Pydantic models\n@pydantic.dataclasses.dataclass\nclass MyProperties:\n    foo: str\n\ndata = {\n    "type": "Feature",\n    "geometry": {"type": "Point", "coordinates": [1, 2]},\n    "properties": {"foo": "abc"},\n}\n\n# Parse and validate data into spec-conforming objects\nparsed = Feature[Point, MyProperties](**data)\n"""\nFeature[Point, MyProperties](\n    type=<GeoJSONObjectType.FEATURE: \'Feature\'>, \n    bbox=None, \n    geometry=Point(\n        type=<GeoJSONObjectType.POINT: \'Point\'>, \n        bbox=None, \n        coordinates=(1.0, 2.0)\n    ), \n    properties=MyProperties(foo=\'abc\'), \n    id=None\n)\n"""\n\n# Dump back into JSON using standard Pydantic methods\nparsed.model_dump_json(exclude_unset=True, indent=2)\n"""\n{\n  "type": "Feature",\n  "geometry": {\n    "type": "Point",\n    "coordinates": [\n      1.0,\n      2.0\n    ]\n  },\n  "properties": {\n    "foo": "abc"\n  }\n}\n"""\n```\n\nParse objects of arbitrary types:\n\n```python\nfrom geodantic import GeometryCollection\n\ndata = {\n    "type": "GeometryCollection",\n    "geometries": [\n        {\n            "type": "Polygon",\n            "coordinates": [[[1, 2], [3, 4], [5, 6], [1, 2]]],\n            "bbox": [1, 2, 3, 4],\n        },\n        {\n            "type": "GeometryCollection",\n            "geometries": [{"type": "Point", "coordinates": [1, 2]}],\n        },\n    ],\n}\n\n# Parse any geometry type\nparsed = GeometryCollection(**data)\n"""\nGeometryCollection(\n    type=<GeoJSONObjectType.GEOMETRY_COLLECTION: \'GeometryCollection\'>, \n    bbox=None, \n    geometries=[\n        Polygon(\n            type=<GeoJSONObjectType.POLYGON: \'Polygon\'>, \n            bbox=(1.0, 2.0, 3.0, 4.0), \n            coordinates=[[(1.0, 2.0), (3.0, 4.0), (5.0, 6.0), (1.0, 2.0)]]\n        ), \n        GeometryCollection(\n            type=<GeoJSONObjectType.GEOMETRY_COLLECTION: \'GeometryCollection\'>, \n            bbox=None, \n            geometries=[\n                Point(\n                    type=<GeoJSONObjectType.POINT: \'Point\'>, \n                    bbox=None, \n                    coordinates=(1.0, 2.0)\n                )\n            ]\n        )\n    ]\n)\n"""\n```\n\n## Contributing\n\nSet up the project using [Poetry](https://python-poetry.org/):\n\n```\npoetry install\n```\n\nFormat the code:\n\n```\nmake lint\n```\n\nRun tests:\n\n```\nmake test\n```\n\nCheck for typing and format issues:\n\n```\nmake check\n```\n',
    'author': 'Alexander Malyga',
    'author_email': 'alexander@malyga.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/alexandermalyga/geodantic',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.12,<4.0',
}


setup(**setup_kwargs)
